#include <stdlib.h>
#include <stdio.h>
#include "dag.h"

DagNode *
makeDagNode (Triangle * t)
{
  DagNode *node = malloc (sizeof (DagNode));
  #ifdef DEBUG
  	printf("allocating dag children %x\n", node);
  #endif
  node->t = t;
  node->children = NULL;
  node->n = 0;
  node->visited = 0;
  return node;
}

void
append2Children (DagNode * node, DagNode * c1, DagNode * c2)
{
  node->children = malloc (sizeof (DagNode) * 2);
  node->children[0] = c1;
  node->children[1] = c2;
  node->n = 2;
}

void
append3Children (DagNode * node, DagNode * c1, DagNode * c2, DagNode * c3)
{
  node->children = malloc (sizeof (DagNode) * 3);
  node->children[0] = c1;
  node->children[1] = c2;
  node->children[2] = c3;
  node->n = 3;
}

DagNode *
findTriangle (Vertex * v, DagNode * dag)
{
  if (dag->n == 0)
    {
      return dag;
    }
  else
    {
      int i;
      for (i = 0; i < dag->n; i++)
	{
	  if (inTriangle
	      (dag->children[i]->t->v1->point, dag->children[i]->t->v2->point,
	       dag->children[i]->t->v3->point, v->point) == 1)
	    return findTriangle (v, dag->children[i]);
	}
      return NULL;
    }
}

int
getTriangulation (DagNode * node, Triangle ** triangles)
{
  static int n = 0;
  if (node->visited == 0) {
	#ifdef DEBUG
		printf("step %d finding node %x\n", n, node);
		printf("node %d %d\n", node->n, node->visited);
	#endif
  if (node->n == 0)
    {
      if (boundaryTriangle (node->t) == 0)
		triangles[n++] = node->t;
    }
  else
    {
      int i;
      for (i = 0; i < node->n; i++)
		{
	  		getTriangulation (node->children[i], triangles);
		}
    }
  node->visited = 1;
  return n;
  }
}

int
boundaryTriangle (Triangle * t)
{
  return t->v1->artificial + t->v2->artificial + t->v3->artificial;
}

void freeDag(DagNode *node)
{
	if (node == NULL) return;
	if (node->n == 0) {
		if (node->t == NULL) return;
		freeTriangle(node->t);
		node->t = NULL;

		if (node->children == NULL) return;
		#ifdef DEBUG
			printf("freeing dag children %x\n", node->children);
		#endif
		free(node->children);
		node->children = NULL;
		node->n = 0;
		
		if (node == NULL) return;
		#ifdef DEBUG
			printf("freeing dag node %x\n", node);
		#endif
		free(node);
		node = NULL;	
	} else {
		int i;
		if (node-> n == 2)
		{
			freeDag(node->children[0]);
			
			if (node->t == NULL) return;
			freeTriangle(node->t);
			free(node->children);
			free(node);
			node->t = NULL;
			node->children = NULL;
			node->n = 0;
		}
		else
		{
			for(i = 0; i < node->n; i++)
			{
				freeDag(node->children[i]);
			}
			if (node->children == NULL) return;
			#ifdef DEBUG
				printf("freeing dag children %x\n", node->children);
			#endif
			free(node->children);
			node->children = NULL;
			node->n = 0;
		}
	}

}
