#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "topology.h"
#include "dag.h"
#include "svg.h"

void
swapEdgeTriangles (Edge * e, Triangle * o, Triangle * n)
{
  if (e->t1 == o)
    e->t1 = n;
  else
    e->t2 = n;
}

// dado un triangulo t=(v1,v2,v3) e vertice v
void
split (Vertex * v, DagNode * dag)
{
// criar as arestas dos novos triangulos
  Edge *e1 = makeEdge (dag->t->v1, v);
  Edge *e2 = makeEdge (dag->t->v2, v);
  Edge *e3 = makeEdge (dag->t->v3, v);
		
	printSplit(e1, e2, e3);		
// criar os triangulos v1,v,v2; v1,v,v3; v2,v,v3;
  Triangle *t1 = makeTriangle (dag->t->v1, v, dag->t->v2);
  t1->e1 = e1;
  t1->e2 = e2;
  t1->e3 = getTriangleEdge (dag->t->v1, dag->t->v2, dag->t);

  Triangle *t2 = makeTriangle (dag->t->v1, v, dag->t->v3);
  t2->e1 = e1;
  t2->e2 = e3;
  t2->e3 = getTriangleEdge (dag->t->v1, dag->t->v3, dag->t);

  Triangle *t3 = makeTriangle (dag->t->v2, v, dag->t->v3);
  t3->e1 = e2;
  t3->e2 = e3;
  t3->e3 = getTriangleEdge (dag->t->v2, dag->t->v3, dag->t);
  // arruma as adjacencias das arestas
  e1->t1 = t1;
  e1->t2 = t2;
  e2->t1 = t1;
  e2->t2 = t3;
  e3->t1 = t2;
  e3->t2 = t3;

  swapEdgeTriangles (t1->e3, dag->t, t1);
  swapEdgeTriangles (t2->e3, dag->t, t2);
  swapEdgeTriangles (t3->e3, dag->t, t3);

  DagNode *c1 = makeDagNode (t1);
  DagNode *c2 = makeDagNode (t2);
  DagNode *c3 = makeDagNode (t3);

  append3Children (dag, c1, c2, c3);
}


void
flip (Edge * e, DagNode * root)
{
  // Seja t1 e t2 os triângulos opostos a aresta e
  // dados os pontos v1 e v2 de t1 e t2 que não são da aresta e
  // e seja ve1 e ve2 os vértices da aresta e
  Vertex *v1, *v2;
  v1 = getOpposedVertex (e, e->t1);
  v2 = getOpposedVertex (e, e->t2);


  if (v1 == NULL || v2 == NULL)
    return;

  // Adicionar a aresta v1 e v2 na lista de aresta e remover a aresta e da lista de aresta
  Edge *ne = makeEdge (v1, v2);

	printFlip(e, ne);

  // Adicionar os triângulos nt1 (v1, v2, ve1) e nt2 (v1, v2, ve2)
  Triangle *t1 = makeTriangle (v1, v2, e->v1);
  Triangle *t2 = makeTriangle (v1, v2, e->v2);

  Edge *d1 = getTriangleEdge (v2, e->v1, e->t1);
  Edge *d2 = getTriangleEdge (v1, e->v1, e->t2);
  Edge *d3 = getTriangleEdge (v1, e->v2, e->t2);
  Edge *d4 = getTriangleEdge (v2, e->v2, e->t1);

  t1->e1 = ne;
  t1->e2 = d1;
  t1->e3 = d2;

  t2->e1 = ne;
  t2->e2 = d4;
  t2->e3 = d3;

  ne->t1 = t1;
  ne->t2 = t2;

  // Corrigir a vizinhança dos 4 arestas adjacentes a t1 e t2.
  // Os vizinhos da nova aresta criada são, nt1 e nt2
  swapEdgeTriangles (d1, e->t1, t1);
  swapEdgeTriangles (d2, e->t2, t1);
  swapEdgeTriangles (d3, e->t2, t2);
  swapEdgeTriangles (d4, e->t1, t2);

  //Calcular o baricentro do triangulo e->t1 e e->t2 e procurar as folhas as quais pertencem
  Point2 *p1 =
    barycenter (e->t1->v1->point, e->t1->v2->point, e->t1->v3->point);
  Point2 *p2 =
    barycenter (e->t2->v1->point, e->t2->v2->point, e->t2->v3->point);

  Vertex *vp1 = makeVertex (p1);
  Vertex *vp2 = makeVertex (p2);

  DagNode *node1 = findTriangle (vp1, root);
  DagNode *node2 = findTriangle (vp2, root);

  // Libera a memória dos pontos e vértices criados
  freeVertex (vp1);
  freeVertex (vp2);

  DagNode *child1 = makeDagNode (t1);
  DagNode *child2 = makeDagNode (t2);

  append2Children (node1, child1, child2);
  append2Children (node2, child1, child2);

}

// dado o ponto v inserido , uma aresta e, e o ponto oposto o da aresta , testa se aresta é ilegal
int
illegal_edge (Vertex * v, Vertex * o, Edge * e)
{
	int result = 1;

	// Caso seja uma aresta do triângulo artificial ela é legal
	if (e->v2->artificial == 1 && e->v1->artificial == 1)
	{
		result = 0;
	}
	else
	{
		switch (e->v1->artificial + e->v2->artificial + o->artificial + v->artificial)
		{
			case 0:
				result = incircle (*(v->point), *(e->v1->point), *(e->v2->point), *(o->point));
				break;
			case 1:
				result = ((e->v1->artificial == 1) || (e->v2->artificial == 1)) ? 1 : 0;
				if (result == 1) {
					// Testa se é convexo
					if ((leftTurn(*(o->point), *(v->point), *(e->v1->point)) + 
						leftTurn(*(o->point), *(v->point), *(e->v2->point))) % 2 == 0)
						{
							result = 0;
						} 
				}
				break;
			case 2:
				result = (e->v1->point->x > o->point->x) && incircle (*(v->point), *(e->v1->point), *(e->v2->point), *(o->point));
				break;
			default:
				break;
		}
	}

	if (result == 1) printIllegal(e);	
  return result;
}

void
legalize_edge (Vertex * v, Edge * e, Triangle * t, DagNode * root)
{
  // Recupera o vértice o, oposto ao triangulo t na aresta e
  Vertex *o = getOpposedVertex (e, t);

  // Testa se e é ilegal
  if (illegal_edge (v, o, e) == 1)
    {
      // vamos a recuperar as arestas correspondentes a (o, e.v1) e (o.e.v2)
      Triangle *opposed = getOpposedTriangle (e, t);
      Edge *eo1 = NULL;
      Edge *eo2 = NULL;
      if (opposed->e1 == e)
	{
	  eo1 = opposed->e2;
	  eo2 = opposed->e3;
	}
      else if (opposed->e2 == e)
	{
	  eo1 = opposed->e1;
	  eo2 = opposed->e3;
	}
      else if (opposed->e3 == e)
	{
	  eo1 = opposed->e1;
	  eo2 = opposed->e2;
	}

      // Se for, faz o flip de e na triangulação tri
      flip (e, root);

      // Legaliza a aresta (o, e.v1) e (o.e.v2) a partir do vértice v
      legalize_edge (v, eo1, getTriangle (eo1, v), root);
      legalize_edge (v, eo2, getTriangle (eo2, v), root);
    }
}

DagNode *
createArtifitialPoints (Point2 ** S, int n, DagNode * dag)
{
  int i;
  double max = 0;
// barrer de 0 a n os pontos de S e guardar a máxima coordenada encontrada
  for (i = 0; i < n; i++)
    {
      if (fabs (S[i]->x) > max)
	max = fabs (S[i]->x);
      if (fabs (S[i]->y) > max)
	max = fabs (S[i]->y);
    }
  //Criamos os pontos p1 p2 p3 artificiais
  Point2 *p1, *p2, *p3;
  p1 = makePoint2 (3 * max + 1, 0);
  p2 = makePoint2 (0, 3 * max + 1);
  p3 = makePoint2 (-1 * (3 * max + 1), -1 * (3 * max + 1));

  //criar os vertices P1 P2 P3 
  Vertex *v1 = makeVertex (p1);
  Vertex *v2 = makeVertex (p2);
  Vertex *v3 = makeVertex (p3);
  
  v1->artificial = 1;
  v2->artificial = 1;
  v3->artificial = 1;
  
  //Criando as arestas definidas por P1 P2 e P3 

  Edge *e1 = makeEdge (v1, v2);
  Edge *e2 = makeEdge (v2, v3);
  Edge *e3 = makeEdge (v1, v3);

  //Criando o triangulo P1P2P3 e arruma as vizinhanças
  Triangle *t = makeTriangle (v1, v2, v3);


  t->e1 = e1;
  t->e2 = e2;
  t->e3 = e3;

  e1->t1 = t;
  e2->t1 = t;
  e3->t1 = t;
	
	printArtificialTriangle(t);
  //Adicionando o triangulo P1P2P3 à estrutura DAG
  DagNode *root = makeDagNode (t);
  return root;
}
